;;; -*- Mode: Lisp; Package: CALC -*-
(in-package "CALC")  ;; creates package for abstract syntax. 

(in-package "CALC")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER CALC-KEYWORD-LIST
              '(SBST::|,| SBST::WHERE SBST::|,| SBST::SUM SBST::- SBST::/
                SBST::* SBST::- SBST::+ SBST::IN SBST::|,| SBST::LET
                SBST::UNTIL SBST::FROM SBST::ITER SBST::|:| SBST::=))
(DEFPARAMETER CALC-SINGLE-CHAR-OP-LIST
              '(#\, #\+ #\- #\* #\/ #\= #\:))
(DEFPARAMETER CALC-MULTI-CHAR-OP-LIST NIL)
(DEFPARAMETER CALC-ALL-OPERATORS-LIST
              '(SBST::|,| SBST::+ SBST::- SBST::* SBST::/ SBST::= SBST::|:|))
(DEFPARAMETER CALC-NEW-LINE-COMMENT-CHAR #\%)
(DEFPARAMETER CALC-OPEN-COMMENT-CHAR NIL)
(DEFPARAMETER CALC-CLOSE-COMMENT-CHAR NIL)
(DEFPARAMETER CALC-ESCAPE-CHAR NIL)
(DEFPARAMETER CALC-CASE-SENSITIVE NIL)
(DEFPARAMETER CALC-STRING-CHAR NIL)
(DEFPARAMETER CALC-KEYWORD-CHAR NIL)
(DEFPARAMETER CALC-LITERAL-CHAR NIL)
(DEFPARAMETER CALC-RESTRICTED-CHARS
              (REDUCE #'(LAMBDA (R S)
                                (UNION R S :TEST #'CHAR=))
                      (LIST CALC-SINGLE-CHAR-OP-LIST
                            (IF CALC-NEW-LINE-COMMENT-CHAR
                                (LIST CALC-NEW-LINE-COMMENT-CHAR))
                            (IF CALC-OPEN-COMMENT-CHAR
                                (LIST CALC-OPEN-COMMENT-CHAR))
                            (IF CALC-CLOSE-COMMENT-CHAR
                                (LIST CALC-CLOSE-COMMENT-CHAR))
                            (IF CALC-ESCAPE-CHAR
                                (LIST CALC-ESCAPE-CHAR))
                            (IF CALC-STRING-CHAR
                                (LIST CALC-STRING-CHAR))
                            (IF CALC-KEYWORD-CHAR
                                (LIST CALC-KEYWORD-CHAR))
                            (IF CALC-LITERAL-CHAR
                                (LIST CALC-LITERAL-CHAR)))))
(DEFVAR *CALC-KEYWORD-TABLE* NIL)
(DEFUN INIT-LEXER-AUX
       (LEXSTREAM)
       (INIT-LEXICAL-READTABLE LEXSTREAM
                               :SINGLE-CHAR-OP-LIST
                               CALC-SINGLE-CHAR-OP-LIST
                               :NEW-LINE-COMMENT-CHAR
                               CALC-NEW-LINE-COMMENT-CHAR
                               :OPEN-COMMENT-CHAR
                               CALC-OPEN-COMMENT-CHAR
                               :ESCAPE-CHAR
                               CALC-ESCAPE-CHAR
                               :MULTI-CHAR-OP-LIST
                               CALC-MULTI-CHAR-OP-LIST))

(DEFUN INIT-LEXER
       (LEXSTREAM)
       (INIT-LEXER-AUX LEXSTREAM)
       (IF CALC-STRING-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               CALC-STRING-CHAR
                               #'READ-SB-STRING))
       (IF CALC-KEYWORD-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               CALC-KEYWORD-CHAR
                               #'READ-KEYWORD-STRING))
       (IF CALC-LITERAL-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               CALC-LITERAL-CHAR
                               #'READ-LITERAL)))

