;;; -*- Mode: Lisp; Package: CALC -*-
(in-package "CALC")  ;; creates package for abstract syntax. 

(in-package "CALC")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '( CALC-PARSE ))

(DEFPARAMETER CALC-ABS-SYN-PACKAGE
              (FIND-PACKAGE "CALC"))

(DEFUN CALC-PARSE
       (&KEY (NT 'CALC::PROGRAM)
             ERROR-THRESHOLD
             ASK-ABOUT-BAD-TOKENS
             (RETURN-ERRORS NIL)
             (STREAM NIL STREAMP)
             STRING
             FILE
             (EXHAUST-STREAM NIL))
       (COND (STREAM)
             (STRING (SETF STREAM
                           (MAKE-STRING-INPUT-STREAM STRING)))
             (FILE (SETF STREAM (OPEN FILE)))
             (T
              (ERROR
                "Must provide an input means -- either :stream, ~
			:string, or :file.")))
       (SETQ *CALC-KEYWORD-TABLE* (INIT-KEYWORD-TABLE CALC-KEYWORD-LIST))
       (LET ((*PARSER-ERROR* NIL)
             (*PARSER-ERROR-COUNT* 0)
             (*PARSER-RETURN-ERRORS* RETURN-ERRORS)
             (*PARSER-ABORT-THRESHOLD* ERROR-THRESHOLD)
             (*ASK-ABOUT-BAD-TOKENS* ASK-ABOUT-BAD-TOKENS)
             (*LEXICAL-STREAM* (OPEN-LEXICAL-STREAM (OPEN-PLACESTREAM STREAM)))
             (*HOLD-A1* NIL)
             (*HOLD-A2* NIL)
             (*HOLD-B1* NIL)
             (*HOLD-B2* NIL)
             (*ABS-SYN-PACKAGE* CALC-ABS-SYN-PACKAGE)
             (*READER-FUN* #'READER)
             (*APPLY-LEX-TERM-CONSTR-FUN* #'APPLY-LEXICAL-TERMINAL-CONSTRUCTOR)
             (*KEYWORD-TABLE* *CALC-KEYWORD-TABLE*)
             (*CLOSE-COMMENT-CHAR* CALC-CLOSE-COMMENT-CHAR)
             (*CASE-SENSITIVE* CALC-CASE-SENSITIVE))
         (INIT-LEXER *LEXICAL-STREAM*)
         (MULTIPLE-VALUE-BIND (ABS-SYNTAX ERROR-STRING
                                          ERROR-ARGS
                                          ERROR-PLACE)
                              (UNWIND-PROTECT
                                (CATCH 'PARSER-ABORT-CATCH
                                       (PROG1
                                         (CASE NT
                                               (CALC::PROGRAM (!PROGRAM! 0))
                                               (CALC::BINDPAIR (!BINDPAIR! 0))
                                               (CALC::ITEXPR (!ITEXPR! 0))
                                               (CALC::EXPR (!EXPR! 0))
                                               (T
                                                 (ERROR
                                                   "Unknown nonterminal ~A."
                                                   NT)))
                                         (WHEN
                                           (OR FILE STRING EXHAUST-STREAM)
                                           (CLET*
                                             (((TOKEN IGNORE PLACE)
                                                (PEEK-FIRST)))
                                             (UNLESS
                                               (OR (EQ TOKEN :EOF)
                                                   (EQ TOKEN 'EOF))
                                               (DO-SYNTAX-ERROR
                                                 "There is garbage at the end of your ~
			      file or string:~%~A"
                                                 PLACE))))))
                                (UNLESS STREAMP
                                        (CLOSE STREAM)))
                              (IF RETURN-ERRORS
                                  (VALUES ABS-SYNTAX
                                          *PARSER-ERROR*
                                          ERROR-PLACE
                                          ERROR-STRING
                                          ERROR-ARGS)
                                  (VALUES ABS-SYNTAX *PARSER-ERROR*)))))


(DEFUN !ITEXPR!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0)
         (ERGO-IGNORE-IF-UNUSED V0)
         (INITIALS-ONLY
           ((LA-MATCH
              ((SBST::LET) (SBST::ITER)
                           (SBST::!ID!)
                           (SBST::!NUMBER!)
                           (SBST::-)
                           (SBST::SUM)))
             (PROGN (CODE-TO-SLOT V0 (!EXPR! 0))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'CALC::ITEXP
                                              (LIST V0)))))
           (T
             (INITIAL-ERROR
               '((SBST::SUM SBST::SUM SBST::- SBST::!NUMBER! SBST::!ID!
                  SBST::ITER SBST::LET)
                 (SBST::- SBST::SUM SBST::- SBST::!NUMBER! SBST::!ID!
                  SBST::ITER SBST::LET)
                 (SBST::!NUMBER! EPSILON SBST::WHERE SBST::/ SBST::* SBST::-
                  SBST::+)
                 (SBST::!ID! SBST::|:| EPSILON SBST::WHERE SBST::/ SBST::*
                  SBST::- SBST::+)
                 (SBST::ITER SBST::!ID!)
                 (SBST::LET SBST::IN SBST::!ID!)))))
         NIL
         V0))


(DEFUN !BINDPAIR!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (LAM ((SBST::= EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXPR! 0))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'CALC::BP
                                              (LIST V1 V2)))))
           (T (INITIAL-ERROR '((SBST::!ID! SBST::=)))))
         NIL
         V0))


(DEFUN !EXPR!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (MTEMP
             V0
             V1
             V2
             V3
             V4
             V5
             V6)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5 V6)
         (INITIALS
           ((LA-MATCH ((SBST::SUM)))
             (PROGN (LAM ((SBST::SUM EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXPR! 0))
                    (CONS-TO-SLOT V2 V3)
                    (GEN-STAR-PARSE
                      ((SBST::|,| SBST::SUM
                                  SBST::-
                                  SBST::!NUMBER!
                                  SBST::!ID!
                                  SBST::ITER
                                  SBST::LET))
                      (PROGN (LAM ((SBST::|,| EPSILON))
                                  (GOBBLE-TOKEN))
                             (CODE-TO-SLOT V3 (!EXPR! 0)))
                      V2
                      V3)
                    (VALUE-TO-SLOT V0 9)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'CALC::SUM
                                              (DS-TEMP-REPT (CK-REPT-REF V2))))))
           ((LA-MATCH ((SBST::-)))
             (PROGN (LAM ((SBST::- EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXPR! 40))
                    (VALUE-TO-SLOT V0 8)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'CALC::NEG
                                              (LIST V2)))))
           ((LA-MATCH ((SBST::LET)))
             (PROGN (LAM ((SBST::LET EPSILON))
                         (GOBBLE-TOKEN))
                    (OPT-PARSE ((SBST::!ID! SBST::=))
                               (PROGN
                                 (DOUBLEPLUS-PARSE ((SBST::!ID! SBST::=))
                                                   (CODE-TO-SLOT V5
                                                                 (!BINDPAIR! 0))
                                                   V4
                                                   V5
                                                   'SBST::|,|)
                                 (AS-TO-SLOT V4
                                             (MAKE-SB-TERM 'CALC::MYLBP
                                                           (DS-TEMP-REPT
                                                             (CK-REPT-REF V4)))))
                               V2)
                    (AS-TO-SLOT V3
                                (IF (= V2 0)
                                    (MAKE-SB-TERM 'CALC::EXPR-NULL-1
                                                  NIL)
                                    (CK-REPT-REF V4)))
                    (LAM ((SBST::IN EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V6 (!EXPR! 0))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'LET
                                              (LIST V3 V6)))))
           ((LA-MATCH ((SBST::ITER)))
             (PROGN (LAM ((SBST::ITER EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (LAM ((SBST::FROM EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXPR! 0))
                    (LAM ((SBST::UNTIL EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V4 (!ITEXPR! 0))
                    (LAM ((SBST::= EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V5))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'CALC::ITER
                                              (LIST V2 V3 V4 V5)))))
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (OPT-PARSE ((SBST::|:| SBST::!NUMBER!))
                               (PROGN
                                 (LAM ((SBST::|:| EPSILON))
                                      (GOBBLE-TOKEN))
                                 (LAM ((SBST::!NUMBER! EPSILON))
                                      (GOBBLE-TO-SLOT V6))
                                 (AS-TO-SLOT V5
                                             (MAKE-SB-TERM 'CALC::DV
                                                           (LIST V6))))
                               V3)
                    (AS-TO-SLOT V4
                                (IF (= V3 0)
                                    (MAKE-SB-TERM 'CALC::EXPR-NULL-2
                                                  NIL)
                                    V5))
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'CALC::VREF
                                              (LIST V2 V4)))))
           ((LA-MATCH ((SBST::!NUMBER!)))
             (PROGN (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 3)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'CALC::CREF
                                              (LIST V1)))))
           (T
             (INITIAL-ERROR
               '((SBST::!NUMBER! EPSILON) (SBST::!ID! EPSILON SBST::|:|)
                 (SBST::ITER SBST::!ID!) (SBST::LET SBST::IN SBST::!ID!)
                 (SBST::- SBST::SUM SBST::- SBST::!NUMBER! SBST::!ID!
                  SBST::ITER SBST::LET)
                 (SBST::SUM SBST::LET SBST::ITER SBST::!ID! SBST::!NUMBER!
                  SBST::- SBST::SUM)))))
         (VALUE-TO-SLOT V0 V1)
         (COND ((>
                  (COND ((MEMBER (PEEK-FIRST)
                                 '(SBST::+ SBST::- SBST::* SBST::/ SBST::WHERE))
                         (SETQ MTEMP (PEEK-FIRST))
                         (CADR
                           (ASSOC (PEEK-FIRST)
                                  '((SBST::/ 30) (SBST::* 30) (SBST::- 20)
                                    (SBST::+ 20) (SBST::WHERE 11)))))
                        (T 0))
                  RBP)
                (LOOP (CLEAN-VARIABLES (V1 V2 V3 V4 V5 V6))
                      (CASE MTEMP
                            (SBST::WHERE
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::WHERE EPSILON))
                                          (GOBBLE-TOKEN))
                                     (DOUBLEPLUS-PARSE ((SBST::!ID! SBST::=))
                                                       (CODE-TO-SLOT V4
                                                                     (!BINDPAIR!
                                                                      0))
                                                       V3
                                                       V4
                                                       'SBST::|,|)
                                     (VALUE-TO-SLOT V0 10)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'CALC::WEAR
                                                               (LIST V2
                                                                     (MAKE-SB-TERM
                                                                      'CALC::WBP
                                                                      (DS-TEMP-REPT
                                                                      (CK-REPT-REF
                                                                      V3))))))))
                            (SBST::/
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::/ EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXPR! 31))
                                     (VALUE-TO-SLOT V0 7)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'CALC::DIV
                                                               (LIST V2 V3)))))
                            (SBST::*
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::* EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXPR! 31))
                                     (VALUE-TO-SLOT V0 6)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'CALC::MUL
                                                               (LIST V2 V3)))))
                            (SBST::-
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::- EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXPR! 21))
                                     (VALUE-TO-SLOT V0 5)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'CALC::SUB
                                                               (LIST V2 V3)))))
                            (SBST::+
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::+ EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXPR! 21))
                                     (VALUE-TO-SLOT V0 4)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'CALC::ADD
                                                               (LIST V2 V3)))))
                            (T
                              (MEDIAL-ERROR
                                '(SBST::+ SBST::- SBST::* SBST::/ SBST::WHERE))))
                      (VALUE-TO-SLOT V0 V1)
                      (WHEN
                        (<=
                          (COND ((MEMBER (PEEK-FIRST)
                                         '(SBST::+ SBST::- SBST::* SBST::/
                                           SBST::WHERE))
                                 (SETQ MTEMP (PEEK-FIRST))
                                 (CADR
                                   (ASSOC (PEEK-FIRST)
                                          '((SBST::/ 30) (SBST::* 30)
                                            (SBST::- 20) (SBST::+ 20)
                                            (SBST::WHERE 11)))))
                                (T 0))
                          RBP)
                        (RETURN NIL)))))
         V0))


(DEFUN !PROGRAM!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0)
         (ERGO-IGNORE-IF-UNUSED V0)
         (INITIALS-ONLY
           ((LA-MATCH
              ((SBST::LET) (SBST::ITER)
                           (SBST::!ID!)
                           (SBST::!NUMBER!)
                           (SBST::-)
                           (SBST::SUM)))
             (PROGN (CODE-TO-SLOT V0 (!EXPR! 0))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'PROG (LIST V0)))))
           (T
             (INITIAL-ERROR
               '((SBST::SUM SBST::SUM SBST::- SBST::!NUMBER! SBST::!ID!
                  SBST::ITER SBST::LET)
                 (SBST::- SBST::SUM SBST::- SBST::!NUMBER! SBST::!ID!
                  SBST::ITER SBST::LET)
                 (SBST::!NUMBER! EPSILON SBST::WHERE SBST::/ SBST::* SBST::-
                  SBST::+)
                 (SBST::!ID! SBST::|:| EPSILON SBST::WHERE SBST::/ SBST::*
                  SBST::- SBST::+)
                 (SBST::ITER SBST::!ID!)
                 (SBST::LET SBST::IN SBST::!ID!)))))
         NIL
         V0))

