;;; -*- Mode: Lisp; Package: CALC -*-
(in-package "CALC")  ;; creates package for abstract syntax. 

(in-package "CALC")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER *CALC-OPSIG-TABLE*
              (MAKE-OPSIG-TABLE
                '((#^LET
                   . #@("CALC" := (#t(:SORT EXPR-OPT-0) #t(:SORT EXPR)) :-> #t(:SORT EXPR)))
                  (#^PROG . #@("CALC" := (#t(:SORT EXPR)) :-> #t(:SORT PROGRAM)))
                  (#^CREF . #@("CALC" := (#t(:SORT NUMBER)) :-> #t(:SORT EXPR)))
                  (#^VREF
                   . #@("CALC" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXPR-OPT-1)) :-> #t(:SORT EXPR)))
                  (#^WEAR
                   . #@("CALC" := (#t(:SORT EXPR) #t(:SORT WBP)) :-> #t(:SORT EXPR)))
                  (#^EXPR-NULL-2
                   . #@("CALC" := (#t(:NULL)) :-> #t(:SORT EXPR-OPT-1)))
                  (#^MUL
                   . #@("CALC" := (#t(:SORT EXPR) #t(:SORT EXPR)) :-> #t(:SORT EXPR)))
                  (#^ADD
                   . #@("CALC" := (#t(:SORT EXPR) #t(:SORT EXPR)) :-> #t(:SORT EXPR)))
                  (#^EXPR-NULL-1
                   . #@("CALC" := (#t(:NULL)) :-> #t(:SORT EXPR-OPT-0)))
                  (#^DIV
                   . #@("CALC" := (#t(:SORT EXPR) #t(:SORT EXPR)) :-> #t(:SORT EXPR)))
                  (#^ITER
                   . #@("CALC" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXPR) #t(:SORT ITEXPR) #t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT EXPR)))
                  (#^SUM
                   . #@("CALC" := (#t(:LIST (:SORT EXPR))) :-> #t(:SORT EXPR)))
                  (#^NEG . #@("CALC" := (#t(:SORT EXPR)) :-> #t(:SORT EXPR)))
                  (#^SUB
                   . #@("CALC" := (#t(:SORT EXPR) #t(:SORT EXPR)) :-> #t(:SORT EXPR)))
                  (#^MYLBP
                   . #@("CALC" := (#t(:LIST (:SORT BINDPAIR))) :-> #t(:SORT EXPR-OPT-0)))
                  (#^WBP
                   . #@("CALC" := (#t(:LIST (:SORT BINDPAIR))) :-> #t(:SORT WBP)))
                  (#^ITEXP . #@("CALC" := (#t(:SORT EXPR)) :-> #t(:SORT ITEXPR)))
                  (#^BP
                   . #@("CALC" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXPR)) :-> #t(:SORT BINDPAIR)))
                  (#^DV
                   . #@("CALC" := (#t(:SORT NUMBER)) :-> #t(:SORT EXPR-OPT-1))))))


(DEFPARAMETER *CALC-SORT-TABLE*
              (MAKE-SORT-TABLE
                '((#t(:SORT BINDPAIR) . #t(:OP #^BP))
                  (#t(:SORT WBP) . #t(:OP #^WBP))
                  (#t(:SORT PROGRAM) . #t(:OP #^PROG))
                  (#t(:SORT EXPR)
                   . #t(:UNION (:OP #^WEAR) (:OP #^SUM) (:OP #^NEG) (:OP #^DIV) (:OP #^MUL) (:OP #^SUB) (:OP #^ADD) (:OP #^CREF) (:OP #^VREF) (:OP #^ITER) (:OP #^LET)))
                  (#t(:SORT EXPR-OPT-0)
                   . #t(:UNION (:OP #^MYLBP) (:OP #^EXPR-NULL-1)))
                  (#t(:SORT EXPR-OPT-1)
                   . #t(:UNION (:OP #^DV) (:OP #^EXPR-NULL-2)))
                  (#t(:SORT ITEXPR) . #t(:OP #^ITEXP)))))

(lang:lang-define 
:name "CALC"
:conc-name "calc"
:code-package "CALC"
:abs-syn-package "CALC"
:use-packages '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR")
:sub-languages '("LEXICAL-TERMINALS")
:unparse-nts 'T
:parse-routine-name 'CALC::CALC-PARSE
:unparse-routine-name 'CALC::CALC-UNPARSE
:win-unparse-routine-name 'CALC::CALC-WIN-UNPARSE
:sort-table-name 'CALC::*CALC-SORT-TABLE*
:opsig-table-name 'CALC::*CALC-OPSIG-TABLE*
)

