;;; -*- Mode: Lisp; Package: SF -*-
(in-package "SF")  ;; creates package for abstract syntax. 

(in-package "SF")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER SF-KEYWORD-LIST
              '(SBST::= SBST::NOT SBST::AND SBST::=> SBST::TRUE SBST::FALSE
                SBST::* SBST::@ SBST::} SBST::{ SBST::|:=| SBST::SKIP SBST::+
                SBST::|(| SBST::|)|))
(DEFPARAMETER SF-SINGLE-CHAR-OP-LIST
              '(#\+ #\* #\{ #\} #\( #\) #\> #\< #\; #\[ #\] #\@ #\.))
(DEFPARAMETER SF-MULTI-CHAR-OP-LIST
              '((#\| . LEX-\|) (#\: . |LEX-:|) (#\= . LEX-=)))
(DEFPARAMETER SF-ALL-OPERATORS-LIST
              '(SBST::= SBST::=> SBST::+ SBST::* SBST::{ SBST::} SBST::|(|
                SBST::|)| SBST::> SBST::< SBST::|:=| SBST::\|\| SBST::|;|
                SBST::[ SBST::] SBST::@ SBST::|.|))
(DEFPARAMETER SF-NEW-LINE-COMMENT-CHAR #\%)
(DEFPARAMETER SF-OPEN-COMMENT-CHAR NIL)
(DEFPARAMETER SF-CLOSE-COMMENT-CHAR NIL)
(DEFPARAMETER SF-ESCAPE-CHAR NIL)
(DEFPARAMETER SF-CASE-SENSITIVE NIL)
(DEFPARAMETER SF-STRING-CHAR NIL)
(DEFPARAMETER SF-KEYWORD-CHAR NIL)
(DEFPARAMETER SF-LITERAL-CHAR NIL)
(DEFPARAMETER SF-RESTRICTED-CHARS
              (REDUCE #'(LAMBDA (R S)
                                (UNION R S :TEST #'CHAR=))
                      (LIST SF-SINGLE-CHAR-OP-LIST
                            (IF SF-NEW-LINE-COMMENT-CHAR
                                (LIST SF-NEW-LINE-COMMENT-CHAR))
                            (IF SF-OPEN-COMMENT-CHAR
                                (LIST SF-OPEN-COMMENT-CHAR))
                            (IF SF-CLOSE-COMMENT-CHAR
                                (LIST SF-CLOSE-COMMENT-CHAR))
                            (IF SF-ESCAPE-CHAR
                                (LIST SF-ESCAPE-CHAR))
                            (IF SF-STRING-CHAR
                                (LIST SF-STRING-CHAR))
                            (IF SF-KEYWORD-CHAR
                                (LIST SF-KEYWORD-CHAR))
                            (IF SF-LITERAL-CHAR
                                (LIST SF-LITERAL-CHAR)))))
(DEFVAR *SF-KEYWORD-TABLE* NIL)
(DEFUN INIT-LEXER-AUX
       (LEXSTREAM)
       (INIT-LEXICAL-READTABLE LEXSTREAM
                               :SINGLE-CHAR-OP-LIST
                               SF-SINGLE-CHAR-OP-LIST
                               :NEW-LINE-COMMENT-CHAR
                               SF-NEW-LINE-COMMENT-CHAR
                               :OPEN-COMMENT-CHAR
                               SF-OPEN-COMMENT-CHAR
                               :ESCAPE-CHAR
                               SF-ESCAPE-CHAR
                               :MULTI-CHAR-OP-LIST
                               SF-MULTI-CHAR-OP-LIST))

(DEFUN INIT-LEXER
       (LEXSTREAM)
       (INIT-LEXER-AUX LEXSTREAM)
       (IF SF-STRING-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               SF-STRING-CHAR
                               #'READ-SB-STRING))
       (IF SF-KEYWORD-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               SF-KEYWORD-CHAR
                               #'READ-KEYWORD-STRING))
       (IF SF-LITERAL-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               SF-LITERAL-CHAR
                               #'READ-LITERAL)))

(DEFUN LEX-=
       (STREAM SYMBOL)
       (DECLARE (IGNORE SYMBOL))
       (LET (HOLDCHAR)
         (SETF HOLDCHAR
               (LEXICAL-READ-CHAR STREAM :EOF))
         (COND ((CHAR= HOLDCHAR #\>) 'SBST::=>)
               (T (LEXICAL-UNREAD-CHAR STREAM) 'SBST::=))))
(DEFUN |LEX-:|
       (STREAM SYMBOL)
       (DECLARE (IGNORE SYMBOL))
       (LET (HOLDCHAR)
         (SETF HOLDCHAR
               (LEXICAL-READ-CHAR STREAM :EOF))
         (COND ((CHAR= HOLDCHAR #\=) 'SBST::|:=|)
               (T
                (LEXICAL-UNREAD-CHAR STREAM)
                (ILLEGAL-TOKEN-ERROR ":")
                :ILLEGAL-TOKEN))))
(DEFUN LEX-\|
       (STREAM SYMBOL)
       (DECLARE (IGNORE SYMBOL))
       (LET (HOLDCHAR)
         (SETF HOLDCHAR
               (LEXICAL-READ-CHAR STREAM :EOF))
         (COND ((CHAR= HOLDCHAR #\|) 'SBST::\|\|)
               (T
                (LEXICAL-UNREAD-CHAR STREAM)
                (ILLEGAL-TOKEN-ERROR "|")
                :ILLEGAL-TOKEN))))
