;;; -*- Mode: Lisp; Package: SF -*-
(in-package "SF")  ;; creates package for abstract syntax. 

(in-package "SF")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '( SF-PARSE ))

(DEFPARAMETER SF-ABS-SYN-PACKAGE
              (FIND-PACKAGE "SF"))

(DEFUN SF-PARSE
       (&KEY (NT 'ASSERT)
             ERROR-THRESHOLD
             ASK-ABOUT-BAD-TOKENS
             (RETURN-ERRORS NIL)
             (STREAM NIL STREAMP)
             STRING
             FILE
             (EXHAUST-STREAM NIL))
       (COND (STREAM)
             (STRING (SETF STREAM
                           (MAKE-STRING-INPUT-STREAM STRING)))
             (FILE (SETF STREAM (OPEN FILE)))
             (T
              (ERROR
                "Must provide an input means -- either :stream, ~
			:string, or :file.")))
       (SETQ *SF-KEYWORD-TABLE* (INIT-KEYWORD-TABLE SF-KEYWORD-LIST))
       (LET ((*PARSER-ERROR* NIL)
             (*PARSER-ERROR-COUNT* 0)
             (*PARSER-RETURN-ERRORS* RETURN-ERRORS)
             (*PARSER-ABORT-THRESHOLD* ERROR-THRESHOLD)
             (*ASK-ABOUT-BAD-TOKENS* ASK-ABOUT-BAD-TOKENS)
             (*LEXICAL-STREAM* (OPEN-LEXICAL-STREAM (OPEN-PLACESTREAM STREAM)))
             (*HOLD-A1* NIL)
             (*HOLD-A2* NIL)
             (*HOLD-B1* NIL)
             (*HOLD-B2* NIL)
             (*ABS-SYN-PACKAGE* SF-ABS-SYN-PACKAGE)
             (*READER-FUN* #'READER)
             (*APPLY-LEX-TERM-CONSTR-FUN* #'APPLY-LEXICAL-TERMINAL-CONSTRUCTOR)
             (*KEYWORD-TABLE* *SF-KEYWORD-TABLE*)
             (*CLOSE-COMMENT-CHAR* SF-CLOSE-COMMENT-CHAR)
             (*CASE-SENSITIVE* SF-CASE-SENSITIVE))
         (INIT-LEXER *LEXICAL-STREAM*)
         (MULTIPLE-VALUE-BIND (ABS-SYNTAX ERROR-STRING
                                          ERROR-ARGS
                                          ERROR-PLACE)
                              (UNWIND-PROTECT
                                (CATCH 'PARSER-ABORT-CATCH
                                       (PROG1
                                         (CASE NT
                                               (ASSERT (!ASSERT! 0))
                                               (SF::MEMB (!MEMB! 0))
                                               (SF::PRED (!PRED! 0))
                                               (EXP (!EXP! 0))
                                               (SF::COM (!COM! 0))
                                               (T
                                                 (ERROR
                                                   "Unknown nonterminal ~A."
                                                   NT)))
                                         (WHEN
                                           (OR FILE STRING EXHAUST-STREAM)
                                           (CLET*
                                             (((TOKEN IGNORE PLACE)
                                                (PEEK-FIRST)))
                                             (UNLESS
                                               (OR (EQ TOKEN :EOF)
                                                   (EQ TOKEN 'EOF))
                                               (DO-SYNTAX-ERROR
                                                 "There is garbage at the end of your ~
			      file or string:~%~A"
                                                 PLACE))))))
                                (UNLESS STREAMP
                                        (CLOSE STREAM)))
                              (IF RETURN-ERRORS
                                  (VALUES ABS-SYNTAX
                                          *PARSER-ERROR*
                                          ERROR-PLACE
                                          ERROR-STRING
                                          ERROR-ARGS)
                                  (VALUES ABS-SYNTAX *PARSER-ERROR*)))))


(DEFUN !COM!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (STACK-BRACKETS SBST::|(| COM)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (LAM ((SBST::|:=| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXP! 0))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::ASSIGN
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::SKIP)))
             (PROGN (LAM ((SBST::SKIP EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::SKIP (LIST)))))
           (T
             (INITIAL-ERROR
               '((SBST::!ID! SBST::|:=|) (SBST::SKIP EPSILON)
                 (SBST::|(| SBST::|(| SBST::!ID! SBST::SKIP)))))
         (VALUE-TO-SLOT V0 V1)
         (EAT-BRACKETS (SBST::|)| SBST::|(|) COM)
         V0))


(DEFUN !EXP!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (LET (MTEMP
             V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (STACK-BRACKETS SBST::|(| EXP)
         (INITIALS
           ((LA-MATCH ((SBST::!NUMBER!)))
             (PROGN (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::CREF
                                              (LIST V1)))))
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::VREF
                                              (LIST V1)))))
           (T
             (INITIAL-ERROR
               '((SBST::!ID! EPSILON) (SBST::!NUMBER! EPSILON)
                 (SBST::|(| SBST::|(| SBST::!ID! SBST::!NUMBER!)))))
         (VALUE-TO-SLOT V0 V1)
         (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
         (COND ((>
                  (COND ((MEMBER (PEEK-FIRST)
                                 '(SBST::+))
                         (SETQ MTEMP (PEEK-FIRST))
                         (CADR (ASSOC (PEEK-FIRST)
                                      '((SBST::+ 10)))))
                        (T 0))
                  RBP)
                (LOOP (CLEAN-VARIABLES (V1 V2 V3))
                      (CASE MTEMP
                            (SBST::+
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::+ EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 2)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'SF::ADD
                                                               (LIST V2 V3)))))
                            (T (MEDIAL-ERROR '(SBST::+))))
                      (VALUE-TO-SLOT V0 V1)
                      (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
                      (WHEN
                        (<=
                          (COND ((MEMBER (PEEK-FIRST)
                                         '(SBST::+))
                                 (SETQ MTEMP (PEEK-FIRST))
                                 (CADR (ASSOC (PEEK-FIRST)
                                              '((SBST::+ 10)))))
                                (T 0))
                          RBP)
                        (RETURN NIL)))))
         V0))


(DEFUN !PRED!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (LET (MTEMP
             V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (STACK-BRACKETS SBST::|(| PRED)
         (INITIALS
           ((LA-MATCH ((SBST::!NUMBER!) (SBST::!ID!)))
             (PROGN (CODE-TO-SLOT V2
                                  (!EXP! 0 BRACKET-LIST))
                    (LAM ((SBST::= EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXP! 0))
                    (VALUE-TO-SLOT V0 5)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'EQ
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::NOT)))
             (PROGN (LAM ((SBST::NOT EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!PRED! 30))
                    (VALUE-TO-SLOT V0 4)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::NOTT
                                              (LIST V2)))))
           ((LA-MATCH ((SBST::TRUE)))
             (PROGN (LAM ((SBST::TRUE EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::TRUE (LIST)))))
           ((LA-MATCH ((SBST::FALSE)))
             (PROGN (LAM ((SBST::FALSE EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'SF::FALSE (LIST)))))
           (T
             (INITIAL-ERROR
               '((SBST::FALSE EPSILON) (SBST::TRUE EPSILON)
                 (SBST::NOT SBST::|(| SBST::!ID! SBST::!NUMBER! SBST::NOT
                  SBST::FALSE SBST::TRUE)
                 (SBST::!NUMBER! SBST::|)| SBST::+ SBST::=)
                 (SBST::!ID! SBST::|)| SBST::+ SBST::=)
                 (SBST::|(| SBST::|(| SBST::!ID! SBST::!NUMBER! SBST::NOT
                  SBST::FALSE SBST::TRUE)))))
         (VALUE-TO-SLOT V0 V1)
         (EAT-BRACKETS (SBST::|)| SBST::|(|) PRED)
         (COND ((>
                  (COND ((MEMBER (PEEK-FIRST)
                                 '(SBST::=> SBST::AND))
                         (SETQ MTEMP (PEEK-FIRST))
                         (CADR
                           (ASSOC (PEEK-FIRST)
                                  '((SBST::AND 20) (SBST::=> 10)))))
                        (T 0))
                  RBP)
                (LOOP (CLEAN-VARIABLES (V1 V2 V3))
                      (CASE MTEMP
                            (SBST::AND
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::AND EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!PRED! 21))
                                     (VALUE-TO-SLOT V0 3)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'SF::ANDD
                                                               (LIST V2 V3)))))
                            (SBST::=>
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::=> EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!PRED! 11))
                                     (VALUE-TO-SLOT V0 2)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'SF::IMPLIES
                                                               (LIST V2 V3)))))
                            (T (MEDIAL-ERROR '(SBST::=> SBST::AND))))
                      (VALUE-TO-SLOT V0 V1)
                      (EAT-BRACKETS (SBST::|)| SBST::|(|) PRED)
                      (WHEN
                        (<=
                          (COND ((MEMBER (PEEK-FIRST)
                                         '(SBST::=> SBST::AND))
                                 (SETQ MTEMP (PEEK-FIRST))
                                 (CADR
                                   (ASSOC (PEEK-FIRST)
                                          '((SBST::AND 20) (SBST::=> 10)))))
                                (T 0))
                          RBP)
                        (RETURN NIL)))))
         V0))


(DEFUN !MEMB!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (DECLARE (IGNORE BRACKET-LIST))
       (LET (V0
             V1
             V2
             V3
             V4
             V5)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::|(|) (SBST::!ID!)
                                   (SBST::SKIP)))
             (PROGN (CODE-TO-SLOT V1 (!COM! 0))
                    (LAM ((SBST::{ EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!PRED! 0))
                    (LAM ((SBST::} EPSILON))
                         (GOBBLE-TOKEN))
                    (OPT-PARSE
                      ((SBST::|(| SBST::|(| SBST::{)
                        (SBST::{ SBST::TRUE
                                 SBST::FALSE
                                 SBST::NOT
                                 SBST::|(|
                                 SBST::!ID!
                                 SBST::!NUMBER!))
                      (PROGN (CODE-TO-SLOT V5 (!ASSERT! 0))
                             (AS-TO-SLOT V5
                                         (MAKE-SB-TERM 'SF::CASSERT
                                                       (LIST V5))))
                      V3)
                    (AS-TO-SLOT V4
                                (IF (= V3 0)
                                    (MAKE-SB-TERM 'SF::MEMB-NULL-1
                                                  NIL)
                                    V5))
                    (AS-TO-SLOT V0
                                (MAKE-SB-TERM 'SF::MEMB
                                              (LIST V1 V2 V4)))))
           (T
             (INITIAL-ERROR
               '((SBST::SKIP SBST::{) (SBST::!ID! SBST::|:=|)
                 (SBST::|(| SBST::|(| SBST::!ID! SBST::SKIP)))))
         NIL
         V0))


(DEFUN !ASSERT!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (DECLARE (IGNORE RBP))
       (LET (V0
             V1
             V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4)
         (STACK-BRACKETS SBST::|(| ASSERT)
         (INITIALS-ONLY
           ((LA-MATCH ((SBST::{)))
             (PROGN (LAM ((SBST::{ EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V1 (!PRED! 0))
                    (LAM ((SBST::} EPSILON))
                         (GOBBLE-TOKEN))
                    (ALT-PARSE
                      ((LA-MATCH ((SBST::|(|) (SBST::!ID!)
                                              (SBST::SKIP)))
                        (VALUE-TO-SLOT V2 2)
                        (DOUBLEPLUS-PARSE
                          ((SBST::|(| SBST::|(| SBST::!ID! SBST::SKIP)
                            (SBST::!ID! SBST::|:=|)
                            (SBST::SKIP SBST::{))
                          (CODE-TO-SLOT V4 (!MEMB! 0))
                          V3
                          V4
                          'SBST::+))
                      ((LA-MATCH ((SBST::@))) (VALUE-TO-SLOT V2 0)
                                              (LAM ((SBST::@ EPSILON))
                                                   (GOBBLE-TOKEN)))
                      ((LA-MATCH ((SBST::*))) (VALUE-TO-SLOT V2 1)
                                              (LAM ((SBST::* EPSILON))
                                                   (GOBBLE-TOKEN)))
                      (T
                        (INITIAL-ERROR
                          '((SBST::|(| SBST::|(| SBST::!ID! SBST::SKIP)
                            (SBST::!ID! SBST::|:=|) (SBST::SKIP SBST::{)
                            (SBST::@ EPSILON) (SBST::* EPSILON)))))
                    (AS-TO-SLOT V0
                                (CASE V2
                                      (2
                                        (MAKE-SB-TERM 'SF::INDUCT
                                                      (LIST V1
                                                            (MAKE-SB-TERM
                                                              'SF::MEMBLIST
                                                              (DS-TEMP-REPT
                                                                (CK-REPT-REF V3))))))
                                      (1 (MAKE-SB-TERM 'SF::TRM
                                                       (LIST V1)))
                                      (0 (MAKE-SB-TERM 'SF::DEAD
                                                       (LIST V1)))))))
           (T
             (INITIAL-ERROR
               '((SBST::{ SBST::TRUE SBST::FALSE SBST::NOT SBST::!NUMBER!
                  SBST::!ID! SBST::|(|)
                 (SBST::|(| SBST::|(| SBST::{)))))
         NIL
         (EAT-BRACKETS (SBST::|)| SBST::|(|)
                       ASSERT)
         V0))

