;;; -*- Mode: Lisp; Package: SF -*-
(in-package "SF")  ;; creates package for abstract syntax. 

(in-package "SF")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER *SF-OPSIG-TABLE*
              (MAKE-OPSIG-TABLE
                '((#^ASSIGN
                   . #@("SF" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXP)) :-> #t(:SORT COM)))
                  (#^NOTT . #@("SF" := (#t(:SORT PRED)) :-> #t(:SORT PRED)))
                  (#^CASSERT
                   . #@("SF" := (#t(:SORT ASSERT)) :-> #t(:SORT MEMB-OPT-0)))
                  (#^CREF . #@("SF" := (#t(:SORT NUMBER)) :-> #t(:SORT EXP)))
                  (#^VREF
                   . #@("SF" := (#t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT EXP)))
                  (#^INDUCT
                   . #@("SF" := (#t(:SORT PRED) #t(:SORT MEMBLIST)) :-> #t(:SORT ASSERT)))
                  (#^ADD
                   . #@("SF" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^MEMB
                   . #@("SF" := (#t(:SORT COM) #t(:SORT PRED) #t(:SORT MEMB-OPT-0)) :-> #t(:SORT MEMB)))
                  (#^MEMBLIST
                   . #@("SF" := (#t(:LIST (:SORT MEMB))) :-> #t(:SORT MEMBLIST)))
                  (#^SKIP . #@("SF" := NIL :-> #t(:SORT COM)))
                  (#^IMPLIES
                   . #@("SF" := (#t(:SORT PRED) #t(:SORT PRED)) :-> #t(:SORT PRED)))
                  (#^TRM . #@("SF" := (#t(:SORT PRED)) :-> #t(:SORT ASSERT)))
                  (#^DEAD . #@("SF" := (#t(:SORT PRED)) :-> #t(:SORT ASSERT)))
                  (#^TRUE . #@("SF" := NIL :-> #t(:SORT PRED)))
                  (#^FALSE . #@("SF" := NIL :-> #t(:SORT PRED)))
                  (#^MEMB-NULL-1
                   . #@("SF" := (#t(:NULL)) :-> #t(:SORT MEMB-OPT-0)))
                  (#^ANDD
                   . #@("SF" := (#t(:SORT PRED) #t(:SORT PRED)) :-> #t(:SORT PRED)))
                  (#^EQ
                   . #@("SF" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT PRED))))))


(DEFPARAMETER *SF-SORT-TABLE*
              (MAKE-SORT-TABLE
                '((#t(:SORT EXP)
                   . #t(:UNION (:OP #^ADD) (:OP #^VREF) (:OP #^CREF)))
                  (#t(:SORT PRED)
                   . #t(:UNION (:OP #^EQ) (:OP #^NOTT) (:OP #^ANDD) (:OP #^IMPLIES) (:OP #^FALSE) (:OP #^TRUE)))
                  (#t(:SORT ASSERT)
                   . #t(:UNION (:OP #^INDUCT) (:OP #^TRM) (:OP #^DEAD)))
                  (#t(:SORT MEMBLIST) . #t(:OP #^MEMBLIST))
                  (#t(:SORT MEMB-OPT-0)
                   . #t(:UNION (:OP #^CASSERT) (:OP #^MEMB-NULL-1)))
                  (#t(:SORT COM) . #t(:UNION (:OP #^ASSIGN) (:OP #^SKIP)))
                  (#t(:SORT MEMB) . #t(:OP #^MEMB)))))

(lang:lang-define 
:name "SF"
:conc-name "sf"
:code-package "SF"
:abs-syn-package "SF"
:use-packages '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR")
:sub-languages '("LEXICAL-TERMINALS")
:unparse-nts 'T
:parse-routine-name 'SF::SF-PARSE
:unparse-routine-name 'SF::SF-UNPARSE
:win-unparse-routine-name 'SF::SF-WIN-UNPARSE
:sort-table-name 'SF::*SF-SORT-TABLE*
:opsig-table-name 'SF::*SF-OPSIG-TABLE*
)

