;;; -*- Mode: Lisp; Package: SF -*-
(in-package "SF")  ;; creates package for abstract syntax. 

(in-package "SF")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '( SF-UNPARSE  SF-WIN-UNPARSE ))

(DEFVAR SF-BRACKET-INFO
        (MAKE-HASH-TABLE :TEST #'EQ))
(CLRHASH SF-BRACKET-INFO)
(MAPC
  #'(LAMBDA (ENTRY)
     (SET-BRACKET-INFO (CAR ENTRY) (CADR ENTRY) (CADDR ENTRY) SF-BRACKET-INFO))
  '((ASSERT SBST::|(| SBST::|)|) (SF::PRED SBST::|(| SBST::|)|)
    (EXP SBST::|(| SBST::|)|) (SF::COM SBST::|(| SBST::|)|)))

(DEFVAR SF-PREC-INFO
        (MAKE-HASH-TABLE :TEST #'EQ))
(CLRHASH SF-PREC-INFO)
(MAPC #'(LAMBDA (NT)
                (INIT-PREC-INFO NT SF-PREC-INFO))
      '(SF::PRED EXP))
(MAPC
  #'(LAMBDA (ENTRY)
     (SET-PREC-INFO (CAR ENTRY) (CADR ENTRY) (CADDR ENTRY) (CADDDR ENTRY)
      SF-PREC-INFO))
  '((SF::PRED SBST::NOT 0 30) (SF::PRED SBST::AND 1 20)
    (SF::PRED SBST::=> 1 10) (SF::PRED SBST::AND 2 21)
    (SF::PRED SBST::=> 2 11) (EXP SBST::+ 1 10) (EXP SBST::+ 2 11)))

(DEFVAR SF-SPACING-INFO
        (MAKE-HASH-TABLE :TEST #'EQ))
(CLRHASH SF-SPACING-INFO)
(MAPC
  #'(LAMBDA (T1 T2 BP)
     (COND ((ASSOC T2 (GETHASH T1 SF-SPACING-INFO)))
      (T
       (SETF (GETHASH T1 SF-SPACING-INFO)
        (CONS (LIST T2 BP) (GETHASH T1 SF-SPACING-INFO))))))
  'NIL
  'NIL
  'NIL)

(DEFVAR SF-KEY-TOKEN-MAP
        (MAKE-HASH-TABLE :TEST #'EQ))
(DEFVAR SF-KEY-ESC-TOKEN-MAP
        (IF SF-ESCAPE-CHAR
            (MAKE-HASH-TABLE :TEST #'EQ)))
(DEFVAR SF-LT-TOKEN-MAP
        (MAKE-HASH-TABLE :TEST #'EQUAL))
(DEFVAR SF-LT-ESC-TOKEN-MAP
        (IF SF-ESCAPE-CHAR
            (MAKE-HASH-TABLE :TEST #'EQUAL)))


(DEFUN SF::SF-UNPARSE
       (NT AS
           &KEY
           (STYLE *UNPARSE-STYLE*))
       (LET* ((*UNPARSER-OP-LIST* SF-ALL-OPERATORS-LIST)
              (*KEY-TOKEN-MAP* SF-KEY-TOKEN-MAP)
              (*KEY-ESC-TOKEN-MAP* SF-KEY-ESC-TOKEN-MAP)
              (*LT-TOKEN-MAP* SF-LT-TOKEN-MAP)
              (*LT-ESC-TOKEN-MAP* SF-LT-ESC-TOKEN-MAP)
              (*APPLY-LT-DIS-FUN* #'APPLY-LEXICAL-TERMINAL-DISCRIMINATOR)
              (*APPLY-LT-DES-FUN* #'APPLY-LEXICAL-TERMINAL-DESTRUCTOR)
              (*APPLY-LT-TOKEN-CONS-FUN* #'APPLY-LT-TOKEN-CONSTRUCTOR)
              (*BRACKET-INFO* SF-BRACKET-INFO)
              (*PREC-INFO* SF-PREC-INFO)
              (SORT (OPSIG-OUTPUT (OPSIG-TABLE-LOOKUP (TERM-OP AS))))
              (NT-NAME
                (COND (NT)
                      ((IS-SORT-TTYPE SORT) (DS-SORT-TTYPE SORT))
                      (T 'ASSERT)))
              (*CASE-SENSITIVE* SF-CASE-SENSITIVE)
              (*ESCAPE-CHARACTER* SF-ESCAPE-CHAR)
              (*RESTRICTED-CHARS* SF-RESTRICTED-CHARS)
              (*STRING-CHAR* SF-STRING-CHAR)
              (*LITERAL-CHAR* SF-LITERAL-CHAR)
              (*KEYWORD-CHAR* SF-KEYWORD-CHAR)
              (*UNPARSE-STYLE*
                (IF (AND STYLE
                         (NOT (CONSP STYLE)))
                    (LIST STYLE)
                    STYLE))
              (*NO-ESCAPES* (OR *NO-ESCAPES*
                                (NULL (INSERT-ESCAPES?))))
              (*CURRENT-PRINT-DEPTH* 1))
         (LET ((UTERM
                 (CASE NT-NAME
                       (ASSERT (SF-UNP-ASSERT AS :TOP-LEVEL? T))
                       (SF::MEMB (SF-UNP-MEMB AS :TOP-LEVEL? T))
                       (SF::PRED (SF-UNP-PRED AS :TOP-LEVEL? T))
                       (EXP (SF-UNP-EXP AS :TOP-LEVEL? T))
                       (SF::COM (SF-UNP-COM AS :TOP-LEVEL? T))
                       (T (UNPARSE-RUNTIME-ERROR :NT-NOT-UNPARSABLE
                                                 NT-NAME)
                          NIL))))
           UTERM)))

(DEFUN SF::SF-WIN-UNPARSE
       (UTERM WIDTH
              &KEY
              (FONTWIDTH SB-DEFFONTWIDTH)
              (FONTHEIGHT SB-DEFFONTHEIGHT))
       (LET* ((*CASE-SENSITIVE* SF-CASE-SENSITIVE)
              (*UNPARSER-OP-LIST* SF-ALL-OPERATORS-LIST)
              (*SPACING-INFO* SF-SPACING-INFO))
         (FORMAT-UTERM UTERM
                       WIDTH
                       :FONTWIDTH
                       FONTWIDTH
                       :FONTHEIGHT
                       FONTHEIGHT)))



(DEFUN SF-UNP-ASSERT
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'SF-UNP-ASSERT-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN SF-UNP-MEMB
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'SF-UNP-MEMB-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN SF-UNP-PRED
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'SF-UNP-PRED-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN SF-UNP-EXP
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'SF-UNP-EXP-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN SF-UNP-COM
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'SF-UNP-COM-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN SF-UNP-ASSERT-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4)
         (SETF V0 AS)
         (NT-UNP 'ASSERT
                 AS
                 ((UNP-UTERM V0
                             ((UNP-BIND 'SEQ_ V0)
                               (DIS-ALT V2
                                        ((LAMBDA (X)
                                                 (AND (DIS-OP 'SF::DEAD
                                                              X)))
                                          (LAMBDA (X)
                                                  (AND (DIS-OP 'SF::TRM
                                                               X)))
                                          (LAMBDA (X)
                                                  (AND (DIS-OP 'SF::INDUCT
                                                               X)
                                                       (AND
                                                         (DIS-OP 'SF::MEMBLIST
                                                                 (TERM-ARGN X
                                                                      1)))))))
                               (UNP-ALT-AUG V2
                                            (((UNP-TERM-CONST 'SF::DEAD)
                                               (UNP-NAME 'PRED V1))
                                              ((UNP-TERM-CONST 'SF::TRM)
                                                (UNP-NAME 'PRED V1))
                                              ((UNP-TERM-CONST 'SF::INDUCT)
                                                (UNP-NAME 'PRED V1)
                                                (UNP-LIST-CONST)
                                                (UNP-NAME 'DOUBLEPLUS
                                                          V3))))
                               (UNP-SEQ
                                 (((UNP-KEYWORD 'SBST::{))
                                   ((UNP-NT SF-UNP-PRED V1))
                                   ((UNP-KEYWORD 'SBST::}))
                                   ((UNP-ALT V2
                                             (((UNP-KEYWORD 'SBST::@))
                                               ((UNP-KEYWORD 'SBST::*))
                                               ((UNP-UTERM V3
                                                           ((UNP-DOUBLE-REPT V3
                                                                      ((UNP-UTERM
                                                                      V4
                                                                      ((UNP-NT
                                                                      SF-UNP-MEMB
                                                                      V4))))
                                                                      ((UNP-KEYWORD
                                                                      'SBST::+))
                                                                      V4
                                                                      (MAKE-BP
                                                                      :VALUE
                                                                      2
                                                                      :UNITED-FLAG
                                                                      NIL
                                                                      :SPACES
                                                                      NIL
                                                                      :CRS
                                                                      NIL
                                                                      :FORMAT
                                                                      (LIST))
                                                                      (MAKE-BP
                                                                      :VALUE
                                                                      0
                                                                      :UNITED-FLAG
                                                                      NIL
                                                                      :SPACES
                                                                      NIL
                                                                      :CRS
                                                                      NIL
                                                                      :FORMAT
                                                                      (LIST))))))))))
                                 ((MAKE-BP :VALUE
                                           0
                                           :UNITED-FLAG
                                           NIL
                                           :SPACES
                                           NIL
                                           :CRS
                                           NIL
                                           :FORMAT
                                           (LIST))
                                   (MAKE-BP :VALUE
                                            0
                                            :UNITED-FLAG
                                            NIL
                                            :SPACES
                                            NIL
                                            :CRS
                                            NIL
                                            :FORMAT
                                            (LIST))
                                   (MAKE-BP :VALUE
                                            0
                                            :UNITED-FLAG
                                            NIL
                                            :SPACES
                                            NIL
                                            :CRS
                                            NIL
                                            :FORMAT
                                            (LIST))
                                   (MAKE-BP :VALUE
                                            0
                                            :UNITED-FLAG
                                            NIL
                                            :SPACES
                                            NIL
                                            :CRS
                                            NIL
                                            :FORMAT
                                            (LIST))))))))))



(DEFUN SF-UNP-MEMB-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3
             V4
             V5)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4 V5)
         (SETF V0 AS)
         (NT-UNP 'SF::MEMB
                 AS
                 ((UNP-UTERM V0
                             ((UNP-BIND 'SEQ_ V0) (UNP-TERM-CONST 'SF::MEMB)
                                                  (UNP-NAME 'COM V1)
                                                  (UNP-NAME 'PRED V2)
                                                  (UNP-NAME 'OPT V4)
                                                  (UNP-SEQ
                                                    (((UNP-NT SF-UNP-COM
                                                              V1))
                                                      ((UNP-KEYWORD 'SBST::{))
                                                      ((UNP-NT SF-UNP-PRED
                                                               V2))
                                                      ((UNP-KEYWORD 'SBST::}))
                                                      ((UNP-BIND 'OPT_ V4)
                                                        (DIS-OPT V3
                                                                 (LAMBDA (X)
                                                                      (AND
                                                                      (DIS-OP
                                                                      'SF::MEMB-NULL-1
                                                                      X)))
                                                                 (LAMBDA (X)
                                                                      (AND
                                                                      (DIS-OP
                                                                      'SF::CASSERT
                                                                      X))))
                                                        (UNP-OPT-AUG V3
                                                                     ((UNP-NULL-CONST))
                                                                     ((UNP-NAME
                                                                      'NIL
                                                                      V5)))
                                                        (UNP-OPT V3
                                                                 ((UNP-UTERM V5
                                                                      ((UNP-BIND
                                                                      'NONTERMINAL_
                                                                      V5)
                                                                      (UNP-TERM-CONST
                                                                      'SF::CASSERT)
                                                                      (UNP-NAME
                                                                      'ASSERT
                                                                      V5)
                                                                      (UNP-NT
                                                                      SF-UNP-ASSERT
                                                                      V5)))))))
                                                    ((MAKE-BP :VALUE
                                                              0
                                                              :UNITED-FLAG
                                                              NIL
                                                              :SPACES
                                                              NIL
                                                              :CRS
                                                              NIL
                                                              :FORMAT
                                                              (LIST))
                                                      (MAKE-BP :VALUE
                                                               0
                                                               :UNITED-FLAG
                                                               NIL
                                                               :SPACES
                                                               NIL
                                                               :CRS
                                                               NIL
                                                               :FORMAT
                                                               (LIST))
                                                      (MAKE-BP :VALUE
                                                               0
                                                               :UNITED-FLAG
                                                               NIL
                                                               :SPACES
                                                               NIL
                                                               :CRS
                                                               NIL
                                                               :FORMAT
                                                               (LIST))
                                                      (MAKE-BP :VALUE
                                                               0
                                                               :UNITED-FLAG
                                                               NIL
                                                               :SPACES
                                                               NIL
                                                               :CRS
                                                               NIL
                                                               :FORMAT
                                                               (LIST))
                                                      (MAKE-BP :VALUE
                                                               0
                                                               :UNITED-FLAG
                                                               NIL
                                                               :SPACES
                                                               NIL
                                                               :CRS
                                                               NIL
                                                               :FORMAT
                                                               (LIST))))))))))



(DEFUN SF-UNP-PRED-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (SETF V0 AS)
         (NT-UNP 'SF::PRED
                 AS
                 ((UNP-BIND 'ALT_ V0)
                   (DIS-ALT V0
                            ((LAMBDA (X)
                                     (AND (DIS-OP 'SF::TRUE X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::FALSE X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::IMPLIES X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::ANDD X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::NOTT X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'EQ X)))))
                   (UNP-NAME 'NIL V1)
                   (UNP-ALT V0
                            (((SF-UNP-PRED-AUX-ALT-1 V0 V1))
                              ((SF-UNP-PRED-AUX-ALT-2 V0 V1))
                              ((SF-UNP-PRED-AUX-ALT-3 V0 V1))
                              ((SF-UNP-PRED-AUX-ALT-4 V0 V1))
                              ((SF-UNP-PRED-AUX-ALT-5 V0 V1))
                              ((SF-UNP-PRED-AUX-ALT-6 V0 V1))))))))



(DEFUN SF-UNP-PRED-AUX-ALT-1
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'UKEYWORD_ V1) (UNP-TERM-CONST 'SF::TRUE)
                                              (UNP-KEYWORD 'SBST::TRUE)))))



(DEFUN SF-UNP-PRED-AUX-ALT-2
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'UKEYWORD_ V1) (UNP-TERM-CONST 'SF::FALSE)
                                              (UNP-KEYWORD 'SBST::FALSE)))))



(DEFUN SF-UNP-PRED-AUX-ALT-3
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'SF::IMPLIES)
                                         (UNP-NAME 'PRED V2)
                                         (UNP-NAME 'PRED V3)
                                         (UNP-SEQ
                                           (((UNP-NT SF-UNP-PRED V2))
                                             ((UNP-KEYWORD 'SBST::=>))
                                             ((UNP-NT SF-UNP-PRED V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN SF-UNP-PRED-AUX-ALT-4
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'SF::ANDD)
                                         (UNP-NAME 'PRED V2)
                                         (UNP-NAME 'PRED V3)
                                         (UNP-SEQ
                                           (((UNP-NT SF-UNP-PRED V2))
                                             ((UNP-KEYWORD 'SBST::AND))
                                             ((UNP-NT SF-UNP-PRED V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN SF-UNP-PRED-AUX-ALT-5
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'SF::NOTT)
                                         (UNP-NAME 'PRED V2)
                                         (UNP-SEQ
                                           (((UNP-KEYWORD 'SBST::NOT))
                                             ((UNP-NT SF-UNP-PRED V2)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN SF-UNP-PRED-AUX-ALT-6
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'EQ)
                                         (UNP-NAME 'EXP V2)
                                         (UNP-NAME 'EXP V3)
                                         (UNP-SEQ
                                           (((UNP-NT SF-UNP-EXP V2))
                                             ((UNP-KEYWORD 'SBST::=))
                                             ((UNP-NT SF-UNP-EXP V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN SF-UNP-EXP-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (SETF V0 AS)
         (NT-UNP 'EXP
                 AS
                 ((UNP-BIND 'ALT_ V0)
                   (DIS-ALT V0
                            ((LAMBDA (X)
                                     (AND (DIS-OP 'SF::CREF X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::VREF X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::ADD X)))))
                   (UNP-NAME 'NIL V1)
                   (UNP-ALT V0
                            (((SF-UNP-EXP-AUX-ALT-1 V0 V1))
                              ((SF-UNP-EXP-AUX-ALT-2 V0 V1))
                              ((SF-UNP-EXP-AUX-ALT-3 V0 V1))))))))



(DEFUN SF-UNP-EXP-AUX-ALT-1
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'NONTERMINAL_ V1) (UNP-TERM-CONST 'SF::CREF)
                                                 (UNP-NAME 'NUMBER V1)
                                                 (UNP-LT 'SBST::NUMBER
                                                         V1)))))



(DEFUN SF-UNP-EXP-AUX-ALT-2
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'NONTERMINAL_ V1) (UNP-TERM-CONST 'SF::VREF)
                                                 (UNP-NAME 'ID V1)
                                                 (UNP-LT 'SBST::ID V1)))))



(DEFUN SF-UNP-EXP-AUX-ALT-3
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'SF::ADD)
                                         (UNP-NAME 'EXP V2)
                                         (UNP-NAME 'EXP V3)
                                         (UNP-SEQ
                                           (((UNP-NT SF-UNP-EXP V2))
                                             ((UNP-KEYWORD 'SBST::+))
                                             ((UNP-NT SF-UNP-EXP V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN SF-UNP-COM-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (SETF V0 AS)
         (NT-UNP 'SF::COM
                 AS
                 ((UNP-BIND 'ALT_ V0)
                   (DIS-ALT V0
                            ((LAMBDA (X)
                                     (AND (DIS-OP 'SF::SKIP X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'SF::ASSIGN X)))))
                   (UNP-NAME 'NIL V1)
                   (UNP-ALT V0
                            (((SF-UNP-COM-AUX-ALT-1 V0 V1))
                              ((SF-UNP-COM-AUX-ALT-2 V0 V1))))))))



(DEFUN SF-UNP-COM-AUX-ALT-1
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'UKEYWORD_ V1) (UNP-TERM-CONST 'SF::SKIP)
                                              (UNP-KEYWORD 'SBST::SKIP)))))



(DEFUN SF-UNP-COM-AUX-ALT-2
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'SF::ASSIGN)
                                         (UNP-NAME 'ID V2)
                                         (UNP-NAME 'EXP V3)
                                         (UNP-SEQ
                                           (((UNP-LT 'SBST::ID V2))
                                             ((UNP-KEYWORD 'SBST::|:=|))
                                             ((UNP-NT SF-UNP-EXP V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN SF-DIS-ASSERT
       (AS)
       (OR (IS-NT-OPSIG? 'ASSERT AS)
           ((LAMBDA (X)
                    (OR ((LAMBDA (X)
                                 (DIS-OP 'SF::INDUCT X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::TRM X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::DEAD X))
                          X)))
             AS)))



(DEFUN SF-DIS-MEMB
       (AS)
       (OR (IS-NT-OPSIG? 'SF::MEMB AS)
           ((LAMBDA (X)
                    (DIS-OP 'SF::MEMB X))
             AS)))



(DEFUN SF-DIS-PRED
       (AS)
       (OR (IS-NT-OPSIG? 'SF::PRED AS)
           ((LAMBDA (X)
                    (OR ((LAMBDA (X) (DIS-OP 'EQ X)) X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::NOTT X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::ANDD X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::IMPLIES X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::FALSE X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::TRUE X))
                          X)))
             AS)))



(DEFUN SF-DIS-EXP
       (AS)
       (OR (IS-NT-OPSIG? 'EXP AS)
           ((LAMBDA (X)
                    (OR ((LAMBDA (X)
                                 (DIS-OP 'SF::ADD X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::VREF X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::CREF X))
                          X)))
             AS)))



(DEFUN SF-DIS-COM
       (AS)
       (OR (IS-NT-OPSIG? 'SF::COM AS)
           ((LAMBDA (X)
                    (OR ((LAMBDA (X)
                                 (DIS-OP 'SF::ASSIGN X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'SF::SKIP X))
                          X)))
             AS)))

