;;; -*- Mode: Lisp; Package: TL -*-
(in-package "TL")  ;; creates package for abstract syntax. 

(in-package "TL")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER TL-KEYWORD-LIST
              '(SBST::OD SBST::DO SBST::WHILE SBST::|;| SBST::|:=| SBST::ELSE
                SBST::THEN SBST::IF SBST::READ SBST::WRITE SBST::- SBST::+
                SBST::TRUE SBST::|(| SBST::|)|))
(DEFPARAMETER TL-SINGLE-CHAR-OP-LIST
              '(#\; #\+ #\- #\( #\)))
(DEFPARAMETER TL-MULTI-CHAR-OP-LIST
              '((#\: . |LEX-:|)))
(DEFPARAMETER TL-ALL-OPERATORS-LIST
              '(SBST::|;| SBST::|:=| SBST::+ SBST::- SBST::|(| SBST::|)|))
(DEFPARAMETER TL-NEW-LINE-COMMENT-CHAR #\%)
(DEFPARAMETER TL-OPEN-COMMENT-CHAR NIL)
(DEFPARAMETER TL-CLOSE-COMMENT-CHAR NIL)
(DEFPARAMETER TL-ESCAPE-CHAR NIL)
(DEFPARAMETER TL-CASE-SENSITIVE NIL)
(DEFPARAMETER TL-STRING-CHAR NIL)
(DEFPARAMETER TL-KEYWORD-CHAR NIL)
(DEFPARAMETER TL-LITERAL-CHAR NIL)
(DEFPARAMETER TL-RESTRICTED-CHARS
              (REDUCE #'(LAMBDA (R S)
                                (UNION R S :TEST #'CHAR=))
                      (LIST TL-SINGLE-CHAR-OP-LIST
                            (IF TL-NEW-LINE-COMMENT-CHAR
                                (LIST TL-NEW-LINE-COMMENT-CHAR))
                            (IF TL-OPEN-COMMENT-CHAR
                                (LIST TL-OPEN-COMMENT-CHAR))
                            (IF TL-CLOSE-COMMENT-CHAR
                                (LIST TL-CLOSE-COMMENT-CHAR))
                            (IF TL-ESCAPE-CHAR
                                (LIST TL-ESCAPE-CHAR))
                            (IF TL-STRING-CHAR
                                (LIST TL-STRING-CHAR))
                            (IF TL-KEYWORD-CHAR
                                (LIST TL-KEYWORD-CHAR))
                            (IF TL-LITERAL-CHAR
                                (LIST TL-LITERAL-CHAR)))))
(DEFVAR *TL-KEYWORD-TABLE* NIL)
(DEFUN INIT-LEXER-AUX
       (LEXSTREAM)
       (INIT-LEXICAL-READTABLE LEXSTREAM
                               :SINGLE-CHAR-OP-LIST
                               TL-SINGLE-CHAR-OP-LIST
                               :NEW-LINE-COMMENT-CHAR
                               TL-NEW-LINE-COMMENT-CHAR
                               :OPEN-COMMENT-CHAR
                               TL-OPEN-COMMENT-CHAR
                               :ESCAPE-CHAR
                               TL-ESCAPE-CHAR
                               :MULTI-CHAR-OP-LIST
                               TL-MULTI-CHAR-OP-LIST))

(DEFUN INIT-LEXER
       (LEXSTREAM)
       (INIT-LEXER-AUX LEXSTREAM)
       (IF TL-STRING-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               TL-STRING-CHAR
                               #'READ-SB-STRING))
       (IF TL-KEYWORD-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               TL-KEYWORD-CHAR
                               #'READ-KEYWORD-STRING))
       (IF TL-LITERAL-CHAR
           (LEXICAL-MAKE-MACRO LEXSTREAM
                               TL-LITERAL-CHAR
                               #'READ-LITERAL)))

(DEFUN |LEX-:|
       (STREAM SYMBOL)
       (DECLARE (IGNORE SYMBOL))
       (LET (HOLDCHAR)
         (SETF HOLDCHAR
               (LEXICAL-READ-CHAR STREAM :EOF))
         (COND ((CHAR= HOLDCHAR #\=) 'SBST::|:=|)
               (T
                (LEXICAL-UNREAD-CHAR STREAM)
                (ILLEGAL-TOKEN-ERROR ":")
                :ILLEGAL-TOKEN))))
