;;; -*- Mode: Lisp; Package: TL -*-
(in-package "TL")  ;; creates package for abstract syntax. 

(in-package "TL")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '( TL-PARSE ))

(DEFPARAMETER TL-ABS-SYN-PACKAGE
              (FIND-PACKAGE "TL"))

(DEFUN TL-PARSE
       (&KEY (NT 'TL::STMT)
             ERROR-THRESHOLD
             ASK-ABOUT-BAD-TOKENS
             (RETURN-ERRORS NIL)
             (STREAM NIL STREAMP)
             STRING
             FILE
             (EXHAUST-STREAM NIL))
       (COND (STREAM)
             (STRING (SETF STREAM
                           (MAKE-STRING-INPUT-STREAM STRING)))
             (FILE (SETF STREAM (OPEN FILE)))
             (T
              (ERROR
                "Must provide an input means -- either :stream, ~
			:string, or :file.")))
       (SETQ *TL-KEYWORD-TABLE* (INIT-KEYWORD-TABLE TL-KEYWORD-LIST))
       (LET ((*PARSER-ERROR* NIL)
             (*PARSER-ERROR-COUNT* 0)
             (*PARSER-RETURN-ERRORS* RETURN-ERRORS)
             (*PARSER-ABORT-THRESHOLD* ERROR-THRESHOLD)
             (*ASK-ABOUT-BAD-TOKENS* ASK-ABOUT-BAD-TOKENS)
             (*LEXICAL-STREAM* (OPEN-LEXICAL-STREAM (OPEN-PLACESTREAM STREAM)))
             (*HOLD-A1* NIL)
             (*HOLD-A2* NIL)
             (*HOLD-B1* NIL)
             (*HOLD-B2* NIL)
             (*ABS-SYN-PACKAGE* TL-ABS-SYN-PACKAGE)
             (*READER-FUN* #'READER)
             (*APPLY-LEX-TERM-CONSTR-FUN* #'APPLY-LEXICAL-TERMINAL-CONSTRUCTOR)
             (*KEYWORD-TABLE* *TL-KEYWORD-TABLE*)
             (*CLOSE-COMMENT-CHAR* TL-CLOSE-COMMENT-CHAR)
             (*CASE-SENSITIVE* TL-CASE-SENSITIVE))
         (INIT-LEXER *LEXICAL-STREAM*)
         (MULTIPLE-VALUE-BIND (ABS-SYNTAX ERROR-STRING
                                          ERROR-ARGS
                                          ERROR-PLACE)
                              (UNWIND-PROTECT
                                (CATCH 'PARSER-ABORT-CATCH
                                       (PROG1
                                         (CASE NT
                                               (TL::STMT (!STMT! 0))
                                               (EXP (!EXP! 0))
                                               (T
                                                 (ERROR
                                                   "Unknown nonterminal ~A."
                                                   NT)))
                                         (WHEN
                                           (OR FILE STRING EXHAUST-STREAM)
                                           (CLET*
                                             (((TOKEN IGNORE PLACE)
                                                (PEEK-FIRST)))
                                             (UNLESS
                                               (OR (EQ TOKEN :EOF)
                                                   (EQ TOKEN 'EOF))
                                               (DO-SYNTAX-ERROR
                                                 "There is garbage at the end of your ~
			      file or string:~%~A"
                                                 PLACE))))))
                                (UNLESS STREAMP
                                        (CLOSE STREAM)))
                              (IF RETURN-ERRORS
                                  (VALUES ABS-SYNTAX
                                          *PARSER-ERROR*
                                          ERROR-PLACE
                                          ERROR-STRING
                                          ERROR-ARGS)
                                  (VALUES ABS-SYNTAX *PARSER-ERROR*)))))


(DEFUN !EXP!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (LET (MTEMP
             V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (STACK-BRACKETS SBST::|(| EXP)
         (INITIALS
           ((LA-MATCH ((SBST::TRUE)))
             (PROGN (LAM ((SBST::TRUE EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'TL::TRUE (LIST)))))
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'TL::VREF
                                              (LIST V1)))))
           ((LA-MATCH ((SBST::!NUMBER!)))
             (PROGN (LAM ((SBST::!NUMBER! EPSILON))
                         (GOBBLE-TO-SLOT V1))
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'TL::CREF
                                              (LIST V1)))))
           (T
             (INITIAL-ERROR
               '((SBST::!NUMBER! EPSILON) (SBST::!ID! EPSILON)
                 (SBST::TRUE EPSILON)
                 (SBST::|(| SBST::|(| SBST::!NUMBER! SBST::!ID! SBST::TRUE)))))
         (VALUE-TO-SLOT V0 V1)
         (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
         (COND ((>
                  (COND ((MEMBER (PEEK-FIRST)
                                 '(SBST::+ SBST::-))
                         (SETQ MTEMP (PEEK-FIRST))
                         (CADR
                           (ASSOC (PEEK-FIRST)
                                  '((SBST::- 10) (SBST::+ 10)))))
                        (T 0))
                  RBP)
                (LOOP (CLEAN-VARIABLES (V1 V2 V3))
                      (CASE MTEMP
                            (SBST::-
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::- EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 4)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'TL::SUB
                                                               (LIST V2 V3)))))
                            (SBST::+
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::+ EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!EXP! 11))
                                     (VALUE-TO-SLOT V0 3)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'TL::ADD
                                                               (LIST V2 V3)))))
                            (T (MEDIAL-ERROR '(SBST::+ SBST::-))))
                      (VALUE-TO-SLOT V0 V1)
                      (EAT-BRACKETS (SBST::|)| SBST::|(|) EXP)
                      (WHEN
                        (<=
                          (COND ((MEMBER (PEEK-FIRST)
                                         '(SBST::+ SBST::-))
                                 (SETQ MTEMP (PEEK-FIRST))
                                 (CADR
                                   (ASSOC (PEEK-FIRST)
                                          '((SBST::- 10) (SBST::+ 10)))))
                                (T 0))
                          RBP)
                        (RETURN NIL)))))
         V0))


(DEFUN !STMT!
       (RBP &OPTIONAL
            (BRACKET-LIST (EMPTY-BRACKET-LIST)))
       (LET (MTEMP
             V0
             V1
             V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4)
         (STACK-BRACKETS SBST::|(| STMT)
         (INITIALS
           ((LA-MATCH ((SBST::WHILE)))
             (PROGN (LAM ((SBST::WHILE EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 0))
                    (LAM ((SBST::DO EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!STMT! 0))
                    (LAM ((SBST::OD EPSILON))
                         (GOBBLE-TOKEN))
                    (VALUE-TO-SLOT V0 5)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'TL::WHILE
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::!ID!)))
             (PROGN (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (LAM ((SBST::|:=| EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!EXP! 0))
                    (VALUE-TO-SLOT V0 0)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'TL::ASSGN
                                              (LIST V2 V3)))))
           ((LA-MATCH ((SBST::IF)))
             (PROGN (LAM ((SBST::IF EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V2 (!EXP! 0))
                    (LAM ((SBST::THEN EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V3 (!STMT! 0))
                    (LAM ((SBST::ELSE EPSILON))
                         (GOBBLE-TOKEN))
                    (CODE-TO-SLOT V4 (!STMT! 0))
                    (VALUE-TO-SLOT V0 1)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'TL::ITE
                                              (LIST V2 V3 V4)))))
           ((LA-MATCH ((SBST::READ)))
             (PROGN (LAM ((SBST::READ EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (VALUE-TO-SLOT V0 2)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'READ (LIST V2)))))
           ((LA-MATCH ((SBST::WRITE)))
             (PROGN (LAM ((SBST::WRITE EPSILON))
                         (GOBBLE-TOKEN))
                    (LAM ((SBST::!ID! EPSILON))
                         (GOBBLE-TO-SLOT V2))
                    (VALUE-TO-SLOT V0 3)
                    (AS-TO-SLOT V1
                                (MAKE-SB-TERM 'WRITE (LIST V2)))))
           (T
             (INITIAL-ERROR
               '((SBST::WRITE SBST::!ID!) (SBST::READ SBST::!ID!)
                 (SBST::IF SBST::TRUE SBST::!ID! SBST::!NUMBER! SBST::|(|)
                 (SBST::!ID! SBST::|:=|)
                 (SBST::WHILE SBST::TRUE SBST::!ID! SBST::!NUMBER! SBST::|(|)
                 (SBST::|(| SBST::|(| SBST::WHILE SBST::WRITE SBST::READ
                  SBST::IF SBST::!ID!)))))
         (VALUE-TO-SLOT V0 V1)
         (EAT-BRACKETS (SBST::|)| SBST::|(|) STMT)
         (COND ((>
                  (COND ((MEMBER (PEEK-FIRST)
                                 '(SBST::|;|))
                         (SETQ MTEMP (PEEK-FIRST))
                         (CADR (ASSOC (PEEK-FIRST)
                                      '((SBST::|;| 10)))))
                        (T 0))
                  RBP)
                (LOOP (CLEAN-VARIABLES (V1 V2 V3 V4))
                      (CASE MTEMP
                            (SBST::|;|
                              (PROGN (SLOT-TO-SLOT V2 V0)
                                     (LAM ((SBST::|;| EPSILON))
                                          (GOBBLE-TOKEN))
                                     (CODE-TO-SLOT V3 (!STMT! 11))
                                     (VALUE-TO-SLOT V0 4)
                                     (AS-TO-SLOT V1
                                                 (MAKE-SB-TERM 'TL::CMPD
                                                               (LIST V2 V3)))))
                            (T (MEDIAL-ERROR '(SBST::|;|))))
                      (VALUE-TO-SLOT V0 V1)
                      (EAT-BRACKETS (SBST::|)| SBST::|(|) STMT)
                      (WHEN
                        (<=
                          (COND ((MEMBER (PEEK-FIRST)
                                         '(SBST::|;|))
                                 (SETQ MTEMP (PEEK-FIRST))
                                 (CADR (ASSOC (PEEK-FIRST)
                                              '((SBST::|;| 10)))))
                                (T 0))
                          RBP)
                        (RETURN NIL)))))
         V0))

