;;; -*- Mode: Lisp; Package: TL -*-
(in-package "TL")  ;; creates package for abstract syntax. 

(in-package "TL")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '())

(DEFPARAMETER *TL-OPSIG-TABLE*
              (MAKE-OPSIG-TABLE
                '((#^READ
                   . #@("TL" := (#t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT STMT)))
                  (#^CREF . #@("TL" := (#t(:SORT NUMBER)) :-> #t(:SORT EXP)))
                  (#^CMPD
                   . #@("TL" := (#t(:SORT STMT) #t(:SORT STMT)) :-> #t(:SORT STMT)))
                  (#^ADD
                   . #@("TL" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^VREF
                   . #@("TL" := (#t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT EXP)))
                  (#^WHILE
                   . #@("TL" := (#t(:SORT EXP) #t(:SORT STMT)) :-> #t(:SORT STMT)))
                  (#^WRITE
                   . #@("TL" := (#t(:SORT SB-RUNTIME::ID)) :-> #t(:SORT STMT)))
                  (#^ASSGN
                   . #@("TL" := (#t(:SORT SB-RUNTIME::ID) #t(:SORT EXP)) :-> #t(:SORT STMT)))
                  (#^TRUE . #@("TL" := NIL :-> #t(:SORT EXP)))
                  (#^SUB
                   . #@("TL" := (#t(:SORT EXP) #t(:SORT EXP)) :-> #t(:SORT EXP)))
                  (#^ITE
                   . #@("TL" := (#t(:SORT EXP) #t(:SORT STMT) #t(:SORT STMT)) :-> #t(:SORT STMT))))))


(DEFPARAMETER *TL-SORT-TABLE*
              (MAKE-SORT-TABLE
                '((#t(:SORT EXP)
                   . #t(:UNION (:OP #^SUB) (:OP #^ADD) (:OP #^CREF) (:OP #^VREF) (:OP #^TRUE)))
                  (#t(:SORT STMT)
                   . #t(:UNION (:OP #^WHILE) (:OP #^CMPD) (:OP #^WRITE) (:OP #^READ) (:OP #^ITE) (:OP #^ASSGN))))))

(lang:lang-define 
:name "TL"
:conc-name "tl"
:code-package "TL"
:abs-syn-package "TL"
:use-packages '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR")
:sub-languages '("LEXICAL-TERMINALS")
:unparse-nts 'T
:parse-routine-name 'TL::TL-PARSE
:unparse-routine-name 'TL::TL-UNPARSE
:win-unparse-routine-name 'TL::TL-WIN-UNPARSE
:sort-table-name 'TL::*TL-SORT-TABLE*
:opsig-table-name 'TL::*TL-OPSIG-TABLE*
)

