;;; -*- Mode: Lisp; Package: TL -*-
(in-package "TL")  ;; creates package for abstract syntax. 

(in-package "TL")  ;; enters package for generated code.  

(use-package '(:ergolisp))  ;; standard primitives 
(use-package '("OPER" "OCC" "TERM" "SORT" "SB-RUNTIME" "LANG" "NEWATTR"))


(export '( TL-UNPARSE  TL-WIN-UNPARSE ))

(DEFVAR TL-BRACKET-INFO
        (MAKE-HASH-TABLE :TEST #'EQ))
(CLRHASH TL-BRACKET-INFO)
(MAPC
  #'(LAMBDA (ENTRY)
     (SET-BRACKET-INFO (CAR ENTRY) (CADR ENTRY) (CADDR ENTRY) TL-BRACKET-INFO))
  '((TL::STMT SBST::|(| SBST::|)|) (EXP SBST::|(| SBST::|)|)))

(DEFVAR TL-PREC-INFO
        (MAKE-HASH-TABLE :TEST #'EQ))
(CLRHASH TL-PREC-INFO)
(MAPC #'(LAMBDA (NT)
                (INIT-PREC-INFO NT TL-PREC-INFO))
      '(TL::STMT EXP))
(MAPC
  #'(LAMBDA (ENTRY)
     (SET-PREC-INFO (CAR ENTRY) (CADR ENTRY) (CADDR ENTRY) (CADDDR ENTRY)
      TL-PREC-INFO))
  '((TL::STMT SBST::|;| 1 10) (TL::STMT SBST::|;| 2 11) (EXP SBST::- 1 10)
    (EXP SBST::+ 1 10) (EXP SBST::- 2 11) (EXP SBST::+ 2 11)))

(DEFVAR TL-SPACING-INFO
        (MAKE-HASH-TABLE :TEST #'EQ))
(CLRHASH TL-SPACING-INFO)
(MAPC
  #'(LAMBDA (T1 T2 BP)
     (COND ((ASSOC T2 (GETHASH T1 TL-SPACING-INFO)))
      (T
       (SETF (GETHASH T1 TL-SPACING-INFO)
        (CONS (LIST T2 BP) (GETHASH T1 TL-SPACING-INFO))))))
  'NIL
  'NIL
  'NIL)

(DEFVAR TL-KEY-TOKEN-MAP
        (MAKE-HASH-TABLE :TEST #'EQ))
(DEFVAR TL-KEY-ESC-TOKEN-MAP
        (IF TL-ESCAPE-CHAR
            (MAKE-HASH-TABLE :TEST #'EQ)))
(DEFVAR TL-LT-TOKEN-MAP
        (MAKE-HASH-TABLE :TEST #'EQUAL))
(DEFVAR TL-LT-ESC-TOKEN-MAP
        (IF TL-ESCAPE-CHAR
            (MAKE-HASH-TABLE :TEST #'EQUAL)))


(DEFUN TL::TL-UNPARSE
       (NT AS
           &KEY
           (STYLE *UNPARSE-STYLE*))
       (LET* ((*UNPARSER-OP-LIST* TL-ALL-OPERATORS-LIST)
              (*KEY-TOKEN-MAP* TL-KEY-TOKEN-MAP)
              (*KEY-ESC-TOKEN-MAP* TL-KEY-ESC-TOKEN-MAP)
              (*LT-TOKEN-MAP* TL-LT-TOKEN-MAP)
              (*LT-ESC-TOKEN-MAP* TL-LT-ESC-TOKEN-MAP)
              (*APPLY-LT-DIS-FUN* #'APPLY-LEXICAL-TERMINAL-DISCRIMINATOR)
              (*APPLY-LT-DES-FUN* #'APPLY-LEXICAL-TERMINAL-DESTRUCTOR)
              (*APPLY-LT-TOKEN-CONS-FUN* #'APPLY-LT-TOKEN-CONSTRUCTOR)
              (*BRACKET-INFO* TL-BRACKET-INFO)
              (*PREC-INFO* TL-PREC-INFO)
              (SORT (OPSIG-OUTPUT (OPSIG-TABLE-LOOKUP (TERM-OP AS))))
              (NT-NAME
                (COND (NT)
                      ((IS-SORT-TTYPE SORT) (DS-SORT-TTYPE SORT))
                      (T 'TL::STMT)))
              (*CASE-SENSITIVE* TL-CASE-SENSITIVE)
              (*ESCAPE-CHARACTER* TL-ESCAPE-CHAR)
              (*RESTRICTED-CHARS* TL-RESTRICTED-CHARS)
              (*STRING-CHAR* TL-STRING-CHAR)
              (*LITERAL-CHAR* TL-LITERAL-CHAR)
              (*KEYWORD-CHAR* TL-KEYWORD-CHAR)
              (*UNPARSE-STYLE*
                (IF (AND STYLE
                         (NOT (CONSP STYLE)))
                    (LIST STYLE)
                    STYLE))
              (*NO-ESCAPES* (OR *NO-ESCAPES*
                                (NULL (INSERT-ESCAPES?))))
              (*CURRENT-PRINT-DEPTH* 1))
         (LET ((UTERM
                 (CASE NT-NAME
                       (TL::STMT (TL-UNP-STMT AS :TOP-LEVEL? T))
                       (EXP (TL-UNP-EXP AS :TOP-LEVEL? T))
                       (T (UNPARSE-RUNTIME-ERROR :NT-NOT-UNPARSABLE
                                                 NT-NAME)
                          NIL))))
           UTERM)))

(DEFUN TL::TL-WIN-UNPARSE
       (UTERM WIDTH
              &KEY
              (FONTWIDTH SB-DEFFONTWIDTH)
              (FONTHEIGHT SB-DEFFONTHEIGHT))
       (LET* ((*CASE-SENSITIVE* TL-CASE-SENSITIVE)
              (*UNPARSER-OP-LIST* TL-ALL-OPERATORS-LIST)
              (*SPACING-INFO* TL-SPACING-INFO))
         (FORMAT-UTERM UTERM
                       WIDTH
                       :FONTWIDTH
                       FONTWIDTH
                       :FONTHEIGHT
                       FONTHEIGHT)))



(DEFUN TL-UNP-STMT
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'TL-UNP-STMT-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN TL-UNP-EXP
       (AS &KEY (TOP-LEVEL? NIL))
       (MEMO-UTERM AS
                   #'TL-UNP-EXP-AUX
                   :TOP-LEVEL?
                   TOP-LEVEL?))



(DEFUN TL-UNP-STMT-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3 V4)
         (SETF V0 AS)
         (NT-UNP 'TL::STMT
                 AS
                 ((UNP-BIND 'ALT_ V0)
                   (DIS-ALT V0
                            ((LAMBDA (X)
                                     (AND (DIS-OP 'TL::ASSGN X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::ITE X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'READ X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'WRITE X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::CMPD X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::WHILE X)))))
                   (UNP-NAME 'NIL V1)
                   (UNP-ALT V0
                            (((TL-UNP-STMT-AUX-ALT-1 V0 V1))
                              ((TL-UNP-STMT-AUX-ALT-2 V0 V1))
                              ((TL-UNP-STMT-AUX-ALT-3 V0 V1))
                              ((TL-UNP-STMT-AUX-ALT-4 V0 V1))
                              ((TL-UNP-STMT-AUX-ALT-5 V0 V1))
                              ((TL-UNP-STMT-AUX-ALT-6 V0 V1))))))))



(DEFUN TL-UNP-STMT-AUX-ALT-1
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V2 V3 V4)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'TL::ASSGN)
                                         (UNP-NAME 'ID V2)
                                         (UNP-NAME 'EXP V3)
                                         (UNP-SEQ
                                           (((UNP-LT 'SBST::ID V2))
                                             ((UNP-KEYWORD 'SBST::|:=|))
                                             ((UNP-NT TL-UNP-EXP V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-STMT-AUX-ALT-2
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V2 V3 V4)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'TL::ITE)
                                         (UNP-NAME 'EXP V2)
                                         (UNP-NAME 'STMT V3)
                                         (UNP-NAME 'STMT V4)
                                         (UNP-SEQ
                                           (((UNP-KEYWORD 'SBST::IF))
                                             ((UNP-NT TL-UNP-EXP V2))
                                             ((UNP-KEYWORD 'SBST::THEN))
                                             ((UNP-NT TL-UNP-STMT V3))
                                             ((UNP-KEYWORD 'SBST::ELSE))
                                             ((UNP-NT TL-UNP-STMT V4)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-STMT-AUX-ALT-3
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V2 V3 V4)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'READ)
                                         (UNP-NAME 'ID V2)
                                         (UNP-SEQ
                                           (((UNP-KEYWORD 'SBST::READ))
                                             ((UNP-LT 'SBST::ID V2)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-STMT-AUX-ALT-4
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V2 V3 V4)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'WRITE)
                                         (UNP-NAME 'ID V2)
                                         (UNP-SEQ
                                           (((UNP-KEYWORD 'SBST::WRITE))
                                             ((UNP-LT 'SBST::ID V2)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-STMT-AUX-ALT-5
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V2 V3 V4)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'TL::CMPD)
                                         (UNP-NAME 'STMT V2)
                                         (UNP-NAME 'STMT V3)
                                         (UNP-SEQ
                                           (((UNP-NT TL-UNP-STMT V2))
                                             ((UNP-KEYWORD 'SBST::|;|))
                                             ((UNP-NT TL-UNP-STMT V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-STMT-AUX-ALT-6
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3
             V4)
         (ERGO-IGNORE-IF-UNUSED V2 V3 V4)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'TL::WHILE)
                                         (UNP-NAME 'EXP V2)
                                         (UNP-NAME 'STMT V3)
                                         (UNP-SEQ
                                           (((UNP-KEYWORD 'SBST::WHILE))
                                             ((UNP-NT TL-UNP-EXP V2))
                                             ((UNP-KEYWORD 'SBST::DO))
                                             ((UNP-NT TL-UNP-STMT V3))
                                             ((UNP-KEYWORD 'SBST::OD)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-EXP-AUX
       (AS)
       (LET (V0
             V1
             V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V0 V1 V2 V3)
         (SETF V0 AS)
         (NT-UNP 'EXP
                 AS
                 ((UNP-BIND 'ALT_ V0)
                   (DIS-ALT V0
                            ((LAMBDA (X)
                                     (AND (DIS-OP 'TL::TRUE X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::VREF X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::CREF X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::ADD X)))
                              (LAMBDA (X)
                                      (AND (DIS-OP 'TL::SUB X)))))
                   (UNP-NAME 'NIL V1)
                   (UNP-ALT V0
                            (((TL-UNP-EXP-AUX-ALT-1 V0 V1))
                              ((TL-UNP-EXP-AUX-ALT-2 V0 V1))
                              ((TL-UNP-EXP-AUX-ALT-3 V0 V1))
                              ((TL-UNP-EXP-AUX-ALT-4 V0 V1))
                              ((TL-UNP-EXP-AUX-ALT-5 V0 V1))))))))



(DEFUN TL-UNP-EXP-AUX-ALT-1
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'UKEYWORD_ V1) (UNP-TERM-CONST 'TL::TRUE)
                                              (UNP-KEYWORD 'SBST::TRUE)))))



(DEFUN TL-UNP-EXP-AUX-ALT-2
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'NONTERMINAL_ V1) (UNP-TERM-CONST 'TL::VREF)
                                                 (UNP-NAME 'ID V1)
                                                 (UNP-LT 'SBST::ID V1)))))



(DEFUN TL-UNP-EXP-AUX-ALT-3
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'NONTERMINAL_ V1) (UNP-TERM-CONST 'TL::CREF)
                                                 (UNP-NAME 'NUMBER V1)
                                                 (UNP-LT 'SBST::NUMBER
                                                         V1)))))



(DEFUN TL-UNP-EXP-AUX-ALT-4
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'TL::ADD)
                                         (UNP-NAME 'EXP V2)
                                         (UNP-NAME 'EXP V3)
                                         (UNP-SEQ
                                           (((UNP-NT TL-UNP-EXP V2))
                                             ((UNP-KEYWORD 'SBST::+))
                                             ((UNP-NT TL-UNP-EXP V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-UNP-EXP-AUX-ALT-5
       (V0 V1)
       (ERGO-IGNORE-IF-UNUSED V0 V1)
       (LET (V2
             V3)
         (ERGO-IGNORE-IF-UNUSED V2 V3)
         (UNP-UTERM V1
                    ((UNP-BIND 'SEQ_ V1) (UNP-TERM-CONST 'TL::SUB)
                                         (UNP-NAME 'EXP V2)
                                         (UNP-NAME 'EXP V3)
                                         (UNP-SEQ
                                           (((UNP-NT TL-UNP-EXP V2))
                                             ((UNP-KEYWORD 'SBST::-))
                                             ((UNP-NT TL-UNP-EXP V3)))
                                           ((MAKE-BP :VALUE
                                                     0
                                                     :UNITED-FLAG
                                                     NIL
                                                     :SPACES
                                                     NIL
                                                     :CRS
                                                     NIL
                                                     :FORMAT
                                                     (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))
                                             (MAKE-BP :VALUE
                                                      0
                                                      :UNITED-FLAG
                                                      NIL
                                                      :SPACES
                                                      NIL
                                                      :CRS
                                                      NIL
                                                      :FORMAT
                                                      (LIST))))))))



(DEFUN TL-DIS-STMT
       (AS)
       (OR (IS-NT-OPSIG? 'TL::STMT AS)
           ((LAMBDA (X)
                    (OR ((LAMBDA (X)
                                 (DIS-OP 'TL::WHILE X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::CMPD X))
                          X)
                        ((LAMBDA (X) (DIS-OP 'WRITE X)) X)
                        ((LAMBDA (X) (DIS-OP 'READ X)) X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::ITE X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::ASSGN X))
                          X)))
             AS)))



(DEFUN TL-DIS-EXP
       (AS)
       (OR (IS-NT-OPSIG? 'EXP AS)
           ((LAMBDA (X)
                    (OR ((LAMBDA (X)
                                 (DIS-OP 'TL::SUB X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::ADD X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::CREF X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::VREF X))
                          X)
                        ((LAMBDA (X)
                                 (DIS-OP 'TL::TRUE X))
                          X)))
             AS)))

